﻿using System;
using System.Collections.Generic;

using Ion.HabboHotel.Client.Utilities;
using Ion.HabboHotel.Messenger;

namespace Ion.HabboHotel.Client
{
    public partial class ClientMessageHandler
    {
        private const bool ENABLE_MESSENGER = true;

        /// <summary>
        /// 7 - "@G"
        /// </summary>
        public void InfoRetrieve()
        {
            if (mSession.GetHabbo() != null)
            {
                response.Initialize(ResponseOpcodes.UserObject); // "@E"
                response.AppendObject(mSession.GetHabbo());
                SendResponse();
            }
        }

        /// <summary>
        /// 8 - "@H"
        /// </summary>
        public void GetCredits()
        {
            if (mSession.GetHabbo() != null)
            {
                response.Initialize(ResponseOpcodes.CreditBalance); // "@F"
                response.Append(mSession.GetHabbo().Coins);
                response.AppendString(".0");

                SendResponse();
            }
        }
        /// <summary>
        /// 12 - "@L"
        /// </summary>
        private void MessengerInit()
        {
            if (mSession.InitializeMessenger())
            {
                // Register handlers
                RegisterMessenger();

                // Send initialization message
                response.Initialize(ResponseOpcodes.MessengerInit); // "@L"
                response.writeInt(600);
                response.writeInt(200);
                response.writeInt(600);
                response.writeBool(false);

                response.writeInt(mSession.GetMessenger().GetBuddies().Count);
                foreach (MessengerBuddy buddy in mSession.GetMessenger().GetBuddies())
                {
                    response.AppendObject(buddy);
                }

                SendResponse();
            }
        }
        /// <summary>
        /// 26 - "@Z"
        /// </summary>
        public void ScrGetUserInfo()
        {
            string sSubscription = Request.PopFixedString();
            response.Initialize(ResponseOpcodes.ScrSendUserInfo); // "@G"
            response.writeString(sSubscription);
            response.writeInt(1337);
            response.writeInt(0);
            response.writeInt(0);
            response.writeBool(true);
            SendResponse();
        }

        /// <summary>
        /// 157 - "B]"
        /// </summary>
        public void GetBadges()
        {

        }

        /// <summary>
        /// 233 - "EA"
        /// </summary>
        private void GetIgnoredUsers()
        {
            response.Initialize(ResponseOpcodes.IgnoredUsers); // "Fd"
            response.writeString("Aaron");
            response.writeString("office.boy");
            response.writeString("Phoenix");
            SendResponse();
        }

        /// <summary>
        /// 370 - "Er"
        /// </summary>
        private void GetAchievements()
        {
            // Get achievements from Database
            List<string> achievements = IonEnvironment.GetHabboHotel().GetAchievements().GetAchievements(mSession.GetHabbo().ID);

            // Build response
            response.Initialize(ResponseOpcodes.Achievements); // "Ce"
            response.writeInt(achievements.Count);
            foreach (string achievement in achievements)
            {
                response.writeString(achievement);
            }
            SendResponse();
        }

        /// <summary>
        /// Registers request handlers available from successful login.
        /// </summary>
        public void RegisterUser()
        {
            mRequestHandlers[7] = new RequestHandler(InfoRetrieve);
            mRequestHandlers[8] = new RequestHandler(GetCredits);
            mRequestHandlers[12] = new RequestHandler(MessengerInit);
            mRequestHandlers[26] = new RequestHandler(ScrGetUserInfo);
            mRequestHandlers[157] = new RequestHandler(GetBadges);
            mRequestHandlers[233] = new RequestHandler(GetIgnoredUsers);
            mRequestHandlers[370] = new RequestHandler(GetAchievements);
        }
    }
}
